@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -261,6 +261,73 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -311,57 +378,22 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        if os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false") not in (
-            "true",
-            "false",
-        ):
-            raise ValueError(
-                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
-            )
-        use_client_cert = (
-            os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false") == "true"
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, DataFusionTransport):
             # transport is a DataFusionTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -373,6 +405,15 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -397,6 +438,27 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
         r"""Lists possible versions for Data Fusion instances in
         the specified project and location.
 
+
+        .. code-block:: python
+
+            from google.cloud import data_fusion_v1
+
+            def sample_list_available_versions():
+                # Create a client
+                client = data_fusion_v1.DataFusionClient()
+
+                # Initialize request argument(s)
+                request = data_fusion_v1.ListAvailableVersionsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_available_versions(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.data_fusion_v1.types.ListAvailableVersionsRequest, dict]):
                 The request object. Request message for the list
@@ -426,7 +488,7 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -479,6 +541,27 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
         r"""Lists Data Fusion instances in the specified project
         and location.
 
+
+        .. code-block:: python
+
+            from google.cloud import data_fusion_v1
+
+            def sample_list_instances():
+                # Create a client
+                client = data_fusion_v1.DataFusionClient()
+
+                # Initialize request argument(s)
+                request = data_fusion_v1.ListInstancesRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_instances(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.data_fusion_v1.types.ListInstancesRequest, dict]):
                 The request object. Request message for listing Data
@@ -538,6 +621,25 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
     ) -> datafusion.Instance:
         r"""Gets details of a single Data Fusion instance.
 
+        .. code-block:: python
+
+            from google.cloud import data_fusion_v1
+
+            def sample_get_instance():
+                # Create a client
+                client = data_fusion_v1.DataFusionClient()
+
+                # Initialize request argument(s)
+                request = data_fusion_v1.GetInstanceRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_instance(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.data_fusion_v1.types.GetInstanceRequest, dict]):
                 The request object. Request message for getting details
@@ -590,6 +692,31 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
         r"""Creates a new Data Fusion instance in the specified
         project and location.
 
+
+        .. code-block:: python
+
+            from google.cloud import data_fusion_v1
+
+            def sample_create_instance():
+                # Create a client
+                client = data_fusion_v1.DataFusionClient()
+
+                # Initialize request argument(s)
+                request = data_fusion_v1.CreateInstanceRequest(
+                    parent="parent_value",
+                    instance_id="instance_id_value",
+                )
+
+                # Make the request
+                operation = client.create_instance(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.data_fusion_v1.types.CreateInstanceRequest, dict]):
                 The request object. Request message for creating a Data
@@ -630,7 +757,7 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, instance, instance_id])
         if request is not None and has_flattened_params:
@@ -689,6 +816,29 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
     ) -> operation.Operation:
         r"""Deletes a single Date Fusion instance.
 
+        .. code-block:: python
+
+            from google.cloud import data_fusion_v1
+
+            def sample_delete_instance():
+                # Create a client
+                client = data_fusion_v1.DataFusionClient()
+
+                # Initialize request argument(s)
+                request = data_fusion_v1.DeleteInstanceRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                operation = client.delete_instance(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.data_fusion_v1.types.DeleteInstanceRequest, dict]):
                 The request object. Request message for deleting a Data
@@ -727,7 +877,7 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -783,6 +933,32 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
     ) -> operation.Operation:
         r"""Updates a single Data Fusion instance.
 
+        .. code-block:: python
+
+            from google.cloud import data_fusion_v1
+
+            def sample_update_instance():
+                # Create a client
+                client = data_fusion_v1.DataFusionClient()
+
+                # Initialize request argument(s)
+                instance = data_fusion_v1.Instance()
+                instance.type_ = "DEVELOPER"
+
+                request = data_fusion_v1.UpdateInstanceRequest(
+                    instance=instance,
+                )
+
+                # Make the request
+                operation = client.update_instance(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.data_fusion_v1.types.UpdateInstanceRequest, dict]):
                 The request object. Request message for updating a Data
@@ -828,7 +1004,7 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([instance, update_mask])
         if request is not None and has_flattened_params:
@@ -887,6 +1063,30 @@ class DataFusionClient(metaclass=DataFusionClientMeta):
         r"""Restart a single Data Fusion instance.
         At the end of an operation instance is fully restarted.
 
+
+        .. code-block:: python
+
+            from google.cloud import data_fusion_v1
+
+            def sample_restart_instance():
+                # Create a client
+                client = data_fusion_v1.DataFusionClient()
+
+                # Initialize request argument(s)
+                request = data_fusion_v1.RestartInstanceRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                operation = client.restart_instance(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.data_fusion_v1.types.RestartInstanceRequest, dict]):
                 The request object. Request message for restarting a
